''' NLP Chatbot Deep Learning '''
import nltk # natural language tool kits
import numpy
import tensorflow as tf
import json
import os
from googletrans import Translator

''' 
Loading model and relevant info 
'''    
def load_chat_model() :
    try:
        ''' load trained model '''    
        model = tf.keras.models.load_model('model.chatbot')

        ''' load root words '''
        with open('root-words.json') as f: root_words = json.load(f)

        ''' load intent tags '''
        with open('intent-tags.json') as f: intent_tags = json.load(f)

        ''' 
        load intents file 
        intents => { tag, patterns, responses, context_set } 
        ''' 
        with open('intents.json') as f: dataset = json.load(f)
    except:
        print('*** Config file(s) missing or error!')
        quit()
        
    print("*** Loading model, root words, intent tags, dataset successfully")
    return (model, root_words, intent_tags, dataset)

''' 
Chatting 
'''
def translate(question) :
    trans = Translator()
    x = trans.translate(question)     
    return (x.src, x.text)

def trans_back(lang, answer) :
    trans = Translator()
    x = trans.translate(dest=lang, text=answer)     
    return x.text

def chat(model, root_words, intent_tags, dataset):
    print("Start talking with the bot (type quit to stop)")
    while True:
        lang_question = input("You: ")
        if lang_question.lower() == "quit": break
        
        lang, question = translate(lang_question) 
        
        print("Language: {}  Text: {}".format(lang, question))
        print(root_words) 

        ''' tokenize '''
        #tokens = nltk.word_tokenize(question)
        tokens = nltk.regexp_tokenize(question, "[\w']+")
        print(tokens)

        ''' stemming '''
        stemmer = nltk.stem.LancasterStemmer()
        tokens_tmp = []
        for t in tokens:
            tokens_tmp.append(stemmer.stem(t.lower()))
        print(tokens_tmp)
        
        ''' bag of words '''
        bag_of_words = []
        for w in root_words:
            #found = 1 if w in tokens_tmp else 0
            if w in tokens_tmp :
                found = 1
                print(w)
            else :
                found = 0
            bag_of_words.append(found)
        print(bag_of_words)

        ''' model predict to find a tag '''
        probilities = model.predict(numpy.array([bag_of_words]))
        max_index = numpy.argmax(probilities)
        max_probility = probilities[0][max_index]
        if max_probility < 0.8 :
            print("Sorry, I don't know what you mean. (confidence: {})".format(max_probility))
            print(trans_back(lang, "Sorry, I don't know what you mean."))
            continue
        else :
            found_tag = intent_tags[max_index]

        ''' randomly choose a response according to the found tag '''
        for intent in dataset:
            if intent['tag'] == found_tag: 
                answer = numpy.random.choice(intent['responses'])
                lang_answer = trans_back(lang, answer)
                print("{} (confidence: {})".format(answer, max_probility))
                print("Bot: "+lang_answer)
    
'''
Start up
'''
model, root_words, intent_tags, dataset = load_chat_model()
chat(model, root_words, intent_tags, dataset)